<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| Copyright (c) 2000-2003 the phpAdsNew developers                          |
| For contact details, see: http://www.phpadsnew.com/                       |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: trackeranalysis.plugin.php 3678 2005-09-12 16:16:57Z chris $
*/

/**
 * @package    MaxPlugins
 * @subpackage Reports
 * @author     Misza Pawlowski <michal@m3.net>
 * @author     Radek Maciaszek <radek@m3.net>
 */

require_once MAX_PATH . '/plugins/reports/Reports.php';
require_once MAX_PATH . '/plugins/reports/lib.php';

class Plugins_Reports_Advertiser_Trackeranalysis extends Plugins_Reports {

    // Public info function
    function info()
    {
    	include_once MAX_PATH . '/lib/max/Plugins/PluginTranslation.php';
        Max_PluginTranslation::init($this->module, $this->package);
        
    	$plugininfo = array (
    		"plugin-name"			=> Max_PluginTranslation::translate('Conversion Tracking Analysis', $this->module, $this->package),
    		"plugin-description"	=> Max_PluginTranslation::translate('This report shows a breakdown of all conversion tracking activity.', $this->module, $this->package),
    		'plugin-category'     => 'advertiser',
    		'plugin-category-name'	=> Max_PluginTranslation::translate('Advertiser Reports', $this->module, $this->package),
    		"plugin-author"		=> "Michal Pawlowski",
    		"plugin-export"		=> "csv",
    		"plugin-authorize"	=> phpAds_Admin+phpAds_Agency+phpAds_Client,
    		"plugin-import"		=> array (
    			"clientid"	=> array (
    				"title"		=> $GLOBALS['strClient'],
    				"type"		=> "clientid-dropdown" ),
    			"startdate"	=> array (
                        "title"	=> "Start Date",
                        "type"	=> "edit",
                        "size"	=> 10,
                        "default" => date("Y/m/d",mktime (0,0,0,date("m")-1,1,  date("Y")))
                ),
                "enddate"	=> array (
                        "title"	=> "End Date",
                        "type"	=> "edit",
                        "size"	=> 10,
                        "default" => date("Y/m/d",mktime (0,0,0,date("m"),1,  date("Y")))
                ),
    	   )
        );
    	
    	return ($plugininfo);
    }
    
    /*-------------------------------------------------------*/
    /* Private plugin function                               */
    /*-------------------------------------------------------*/
    
    function execute($clientid, $startdate, $enddate)
    {
        $conf = & $GLOBALS['_MAX']['CONF'];
        require_once 'Spreadsheet/Excel/Writer.php';
            
    	global $date_format;
    	global $phpAds_ThousandsSeperator,$phpAds_DecimalPoint;
    	
    	$reportName = $GLOBALS['strAdvertiserConversionTrackingAnalysisReport'];
    	
    	$startDate = date(str_replace('%','',$date_format), strtotime($startdate));
    	$endDate   = date(str_replace('%','',$date_format), strtotime($enddate));
    	$dbStart = date('Y-m-d', strtotime($startdate));
    	$dbEnd = date('Y-m-d', strtotime($enddate));
    	// creating report object
        $workbook = new Spreadsheet_Excel_Writer();
        //exit;
        $worksheet = generateXLSHeader($workbook, $reportName, $startDate, $endDate,
                                       $clientid, $GLOBALS['strAdvertiserConversionTrackingAnalysisDescription'], 'advertiser');
        
        $worksheet->write(7,0,'Key');
        
        $worksheet->write(8,0,$GLOBALS['strTrackerImpressions']);
        $worksheet->write(8,1,'The total number of impressions that this tracker has for the given time period, regardless of where the viewer came from.');
        
        
        $worksheet->write(9,0,$GLOBALS['strTotalTrackerConnections']);
        $worksheet->write(9,1,'The total number of tracker impressions where the viewer came from a linked media campaign.');
        
        
        $worksheet->write(10,0,$GLOBALS['strTrackerClickConnections']);
        $worksheet->write(10,1,'The total number of tracker connections which resulted from a viewer clicking an ad within the defined campaign click window.');
        
        $worksheet->write(11,0,$GLOBALS['strTrackerImprConnections']);
        $worksheet->write(11,1,'The total number of tracker connections which resulted from a viewer viewing an ad within the defined campaign view window.');
        
        // where to start printing structural data
        $rowStart = 13;
        $columnStart = 0;
        
        // formatting for excel - begin
        $formatInteger =& $workbook->addFormat();
        $formatInteger->setNumFormat($GLOBALS['excell_integer_formatting']);
        
        $formatDecimal =& $workbook->addFormat();
        $formatDecimal->setNumFormat($GLOBALS['excell_decimal_formatting']);
        
        $columnHeader =& $workbook->addFormat();
        $columnHeader->setAlign('center');
        $columnHeader->setBold();
            
        $formatRight =& $workbook->addFormat();
        $formatRight->setAlign('right');
        // formatting for excel - end
        
    
    
        
        $query = "SELECT DATE_FORMAT(diac.tracker_date_time, '%Y-%m-%d') as date
                        ,t.trackername as tracker_name
                        ,t.trackerid as tracker_id
                        ,count(diac.tracker_id) as tracker_impressions
                    FROM {$conf['table']['prefix']}{$conf['table']['trackers']} t
                         LEFT JOIN {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad_connection']} diac
                         ON(
                                t.trackerid = diac.tracker_id
                            AND diac.tracker_date_time > '".$dbStart.' 00:00:00'."'
                            AND diac.tracker_date_time < '".$dbEnd.' 23:59:59'."'
                           )
                    WHERE t.clientid = '$clientid'
                    GROUP BY t.trackerid, t.trackername, date
                    ORDER BY t.tracker_id, diac.tracker_date_time";
        
    	$res = phpAds_dbQuery($query) or phpAds_sqlDie();
    	
    	// if there is any data for this query
    	if (phpAds_dbNumRows($res)) {
        	
        	$previousStart = $rowStart;
        	//var_dump($dataSorted);
        	$i = 0; $currentRow = $rowStart;
        	
        	// getting the db result to the temporary table - results needs to be
        	// prepared first
        	while ($row = phpAds_dbFetchArray($res)) {
        	    $data[] = $row;
        	}
        	    
        	$numrows = (is_array($data)) ? count($data): 0;
            
            // preparing result data
        	$dataSorted = $this->prepareDataImpressions($data);
        	$totals = $this->prepareDataTotals($dataSorted);
        	
        	$currentRow++;
        	
        	$worksheet->write($currentRow, $columnStart,"Tracker Summary", $columnHeader);
        	
        	$currentRow++;
            $worksheet->write($currentRow,$columnStart,
                              $GLOBALS['strName']   , $columnHeader);
            $worksheet->write($currentRow,$columnStart + 1,
                              $GLOBALS['strTrackerImpressions']   , $columnHeader);
            $worksheet->write($currentRow,$columnStart + 2,
                              $GLOBALS['strTotalTrackerConnections']   , $columnHeader);
            $worksheet->write($currentRow,$columnStart + 3,
                              $GLOBALS['strPercentageDecimals'] . '%'   , $columnHeader);
            $worksheet->write($currentRow,$columnStart + 4,
                              $GLOBALS['strTrackerClickConnections']   , $columnHeader);
            $worksheet->write($currentRow,$columnStart + 5,
                              $GLOBALS['strPercentageDecimals'] . '%'   , $columnHeader);
            $worksheet->write($currentRow,$columnStart + 6,
                              $GLOBALS['strTrackerImprConnections']     , $columnHeader);
            $worksheet->write($currentRow,$columnStart + 7,
                              $GLOBALS['strPercentageDecimals'] . '%'   , $columnHeader);
            $currentRow++;
            foreach ($totals as $row) {
                $worksheet->write($currentRow, $columnStart,        $row['tracker_name']);
        	       $worksheet->write($currentRow, $columnStart + 1, (int) $row['tracker_impressions']);
        	       $worksheet->write($currentRow, $columnStart + 2, (int) ($row['connections'][0] + $row['connections'][1]));
        	       $worksheet->write($currentRow, $columnStart + 3, ($row['tracker_impressions'] == 0) ? 0 : ($row['connections'][0] + $row['connections'][1]) / $row['tracker_impressions'] * 100);
        	       $worksheet->write($currentRow, $columnStart + 4, (int)$row['connections'][0]);
        	       $worksheet->write($currentRow, $columnStart + 5, (($row['connections'][0] + $row['connections'][1]) == 0) ? 0 : $row['connections'][0] / ($row['connections'][0] + $row['connections'][1]) * 100 );
        	       $worksheet->write($currentRow, $columnStart + 6, (int) $row['connections'][1]);
        	       $worksheet->write($currentRow, $columnStart + 7, (($row['connections'][0] + $row['connections'][1]) == 0) ? 0 : $row['connections'][1] / ($row['connections'][0] + $row['connections'][1]) * 100 );
        	       $currentRow++;
            }
            
        	$currentRow++;
        	foreach ($dataSorted as $block) {
        	    
        	   // find the priority of this block
        	   if (list(,$row) = each($block)) {
        	       $worksheet->write($currentRow,$columnStart,
        	                         '[id'.$row['tracker_id'].'] '.$row['tracker_name'], $columnHeader);
        	   }
        	   $currentRow++;
        	   
               $worksheet->write($currentRow,$columnStart,
                                 $GLOBALS['strDays']   , $columnHeader);
               $worksheet->write($currentRow,$columnStart + 1,
                                 $GLOBALS['strTrackerImpressions']   , $columnHeader);
               $worksheet->write($currentRow,$columnStart + 2,
                                 $GLOBALS['strTotalTrackerConnections']   , $columnHeader);
               $worksheet->write($currentRow,$columnStart + 3,
                                 $GLOBALS['strPercentageDecimals'] . '%'   , $columnHeader);
               $worksheet->write($currentRow,$columnStart + 4,
                                 $GLOBALS['strTrackerClickConnections']   , $columnHeader);
               $worksheet->write($currentRow,$columnStart + 5,
                                 $GLOBALS['strPercentageDecimals'] . '%'   , $columnHeader);
               $worksheet->write($currentRow,$columnStart + 6,
                                 $GLOBALS['strTrackerImprConnections']     , $columnHeader);
               $worksheet->write($currentRow,$columnStart + 7,
                                 $GLOBALS['strPercentageDecimals'] . '%'   , $columnHeader);
               
               
               $currentRow++;
        	   foreach ($block as $rowId => $row) {
        	       $worksheet->write($currentRow, $columnStart,     ($row['date'] == '') ? '-' : $row['date']);
        	       $worksheet->write($currentRow, $columnStart + 1, (int) $row['tracker_impressions']);
        	       $worksheet->write($currentRow, $columnStart + 2, (int) ($row['connections'][0] + $row['connections'][1]));
        	       $worksheet->write($currentRow, $columnStart + 3, ($row['tracker_impressions'] == 0) ? 0 : ($row['connections'][0] + $row['connections'][1]) / $row['tracker_impressions'] * 100);
        	       $worksheet->write($currentRow, $columnStart + 4, (int)$row['connections'][0]);
        	       $worksheet->write($currentRow, $columnStart + 5, (($row['connections'][0] + $row['connections'][1]) == 0) ? 0 : $row['connections'][0] / ($row['connections'][0] + $row['connections'][1]) * 100 );
        	       $worksheet->write($currentRow, $columnStart + 6, (int) $row['connections'][1]);
        	       $worksheet->write($currentRow, $columnStart + 7, (($row['connections'][0] + $row['connections'][1]) == 0) ? 0 : $row['connections'][1] / ($row['connections'][0] + $row['connections'][1]) * 100 );
        	       $currentRow++;
        	   }
        	   
        	   $currentRow++;
        	   $i++;
        	}
    	} else {
    	    $worksheet->write($rowStart, $columnStart, $GLOBALS['strNoData'], $columnHeader);
    	}
    
        $workbook->close();
    }
    
    
    /*----------------------------------------*/
    /* Function for computing totals          */
    /*----------------------------------------*/
    function prepareDataTotals(& $data)
    {
        $table = array();
        
        $totalRow = array();
        
        foreach($data as $id => $block) {
            foreach($block as $id => $row) {
                $totalRow['tracker_name'] = '[id'.$row['tracker_id'].'] '.$row['tracker_name'];
                $totalRow['tracker_impressions'] += (int) $row['tracker_impressions'];
                $totalRow['connections'][1] += (int) $row['connections'][1];
                $totalRow['connections'][0] += (int) $row['connections'][0];
            }
        }
        
        $table[] = $totalRow;
        
        return $table;
    }
    
    /*----------------------------------------*/
    /* Function for sorting result data       */
    /* and splitting into blocks result array */
    /* for next step - display                */
    /*----------------------------------------*/
    function prepareDataImpressions(& $data)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $retArray = array();
        
        $totalBlock = array();
        $blockNr = 1;
        $previousTracker = '';
        
        // Sorting Array by priority
        foreach ($data as $row) {
            if ($previousTracker != $row['tracker_name']) {
                $previousTracker = $row['tracker_name'];
                $blockIndex = 0;
                $blockNr++;
            }
            
            $query = "SELECT connection_type
                            ,count(t_ad_conn.tracker_id) as connections
                        FROM {$conf['table']['prefix']}{$conf['table']['trackers']} t
                             LEFT JOIN {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad_connection']} t_ad_conn
                             ON (
                                    t.trackerid = t_ad_conn.tracker_id
                                AND tracker_date_time >= '".$row['date']." 00:00:00'
                                AND tracker_date_time < '".$row['date']." 23:59:59'
                                )
                        WHERE t.trackerid = {$row['tracker_id']}
                        GROUP BY connection_type";
            
            $res = phpAds_dbQuery($query) or phpAds_sqlDie();
            
            while ($_row = phpAds_dbFetchArray($res)) {
        	    $row['connections'][$_row['connection_type']] = $_row['connections'];
        	    $row['connection_type'] = $_row['connection_type'];
        	}
    
            $retArray[$blockNr][] = $row;
            $totalBlock[$row['date']]['date']                      = $row['date'];
            $totalBlock[$row['date']]['tracker_impressions']       = $row['tracker_impressions'];
            $blockIndex++;
        }
        
        return $retArray;
    }
    
}

?>